package de.kreawi.trainer;

import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

import javax.swing.UIManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.kreawi.trainer.license.KreawiLicenseHandler;
import de.kreawi.trainer.license.RegistrationHandler;
import de.martinknauer.util.Util;

public class KreawiGlobals {
  
  
  public static final long QUESTION_FILE_SIZE = 9154140;
  public static final String QUESTION_FILE_HASH = "93dbaae0c6a675d8f106f870a14ac567";

  public static final long DEMO_QUESTION_FILE_SIZE = 407941;
  public static final String DEMO_QUESTION_FILE_HASH = "b337395d1f15b9f00422dac7ada6ca4c";

  public static final int CURRENT_YEAR_INT = 2020;
  public static final String CURRENT_YEAR = "" + CURRENT_YEAR_INT;

  public static final String APP_DATA_DIR_NAME_PREFIX = "trainer";

  private static final String APP_DATA_TRAINER_DIR_NAME = APP_DATA_DIR_NAME_PREFIX + KreawiGlobals.CURRENT_YEAR;

  private static final String APP_DATA_KREAWI_DIR_NAME = "kreawi";

  public static final int MINIMUM_IMPORT_YEAR = Math.min(CURRENT_YEAR_INT - 2, 2016);
	
	public static final String INTERNAL_CONFIG_FILE_PATH = "de/kreawi/trainer/resources/trainer.config";

	public static final String CONFIG_FILE_NAME = "trainer.config";

	public static final String RESOURCES_DIR_NAME = "resources";
	
  public static final String APP_IMAGES_DIR_NAME = "resources/images";
  
	public static final String TUTORIAL_DIR_NAME = "tutorial";
  
	public static final String ERROR_IMAGE_FILE_NAME = "fehler.jpg";
	
	public static final String DEMO_IMAGE_LINK_FILE_NAME = "demo_bild.jpg";

	public static final String DEMO_IMAGE_FILE_NAME = "demo_dokument.jpg";
	
	public static final String DEMO_DIR_NAME = "uploads";

	private static Boolean isDemo = null;
	
	private static boolean isDevMode = false;

  private static boolean isBetaMode = false;

	public static final String QUESTION_DEMO_FILE_NAME = "demo.bin";
	
	public static final String QUESTION_DIR_NAME = "data";

	public static final String QUESTION_FILE_NAME = "data.bin";

	public static final String QUESTION_UPDATES_DIR_NAME = "data";

	public static final String QUESTION_UPDATES_FILE_NAME = "update.bin";
	
	public static final String DB_USER = "kreawi";
	public static final String DB_PASSWORD = "supergeheimeslangespasswort";
	public static final String DB_DRIVER_NAME = "org.h2.Driver";
	public static final String DB_DIR_NAME = "db";
  public static final String DB_PREFIX = "jdbc:h2:";
  public static final String DB_NAME = "trainer";
  public static final String DB_SUFFIX = ".mv.db";
  
  private static final String GLOBAL_SETTINGS_DIR = "global_settings";
  private static final String GLOBAL_SETTING_INSTALL_DIR = "install_dir";
  private static final String GLOBAL_SETTING_IS_FULL_VERSION_UPDATE = "is_full_version_update";
  private static final String GLOBAL_SETTING_AVERAGE_STARTUP_TIME = "average_startup_time";
	
  private static final long DEFAULT_AVERAGE_STARTUP_TIME = 5000l;

	public static final Color CORRECT_COLOR = new Color(0, 165, 55);
	
	public static final Color WRONG_COLOR = new Color(165, 0, 0);
	
	public static final float SUCCESS_LIMIT = 0.75f;

  private static File appDir;
  
  private static Logger logger;

  /**
   * Accessing the application data directory.<br>
   * On Windows this will be located in the the user's default application
   * data directory, on other systems the user's home is taken as parent
   * directory.
   * 
   * WARNING: Don't ever use the logger in this function because it has 
   *          to be used before logger configuration is used!!!!
   * 
   * @return the application data directory
   */
  public static File getApplicationDirectory() {
    
    if (appDir == null) {
      
      // for Windows
      String appDirParentPathName = System.getenv("APPDATA");
      String appDirKreawiName = APP_DATA_KREAWI_DIR_NAME;
      
      // for Mac OS X with Sandbox enabled
      if (appDirParentPathName == null && KreawiGlobals.isSandboxEnabled()) {
        appDirParentPathName = System.getProperty("DocumentsDirectory");
      }
      
      // for Mac OS X without Sandbox enabled or Linux
      if (appDirParentPathName == null) {
        appDirParentPathName = System.getProperty("user.home");
        appDirKreawiName = "." + APP_DATA_KREAWI_DIR_NAME;
      }
      
      /* accessing the application data directory */
      appDir = new File(new File(appDirParentPathName, appDirKreawiName), APP_DATA_TRAINER_DIR_NAME);
      appDir.mkdirs();
    }

    return appDir;
  }
  
  /** This is NOT the real file path as there is no file suffix (e.g.: '.mv.db') in the url **/
  public static String getDbUrl() {
    final File dbDirectory = getDbDir();
    dbDirectory.mkdirs();
    final String dbUrl = DB_PREFIX + (new File(dbDirectory, KreawiGlobals.DB_NAME)).getAbsolutePath(); 
    return dbUrl;
  }
  
  /** This is NOT the url to connect to the db. The connect string doesn't want the suffix **/
  public static File getDbFile() {
    final File dbDirectory = getDbDir();
    dbDirectory.mkdirs();
    return new File(dbDirectory, KreawiGlobals.DB_NAME + KreawiGlobals.DB_SUFFIX);
  }

  public static File getDbDir() {
    return new File(KreawiGlobals.getApplicationDirectory(), KreawiGlobals.DB_DIR_NAME);
  }
  
	public static int getCardBoxWarnLimit(final int cardBoxLevel) {
	  final double mult = Math.pow(2, cardBoxLevel);
	  if (mult > Integer.MAX_VALUE) {
	    return Integer.MAX_VALUE;
	  } else {
	    return 100 * (int) mult; 
	  }
	}
	
	public static void configureDefaultUI() {

		try {
			if (System.getProperty("os.name").startsWith("Linux")
					&& UIManager.getSystemLookAndFeelClassName().contains(
							"Metal")) {
				UIManager
						.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
			} else {
				UIManager.setLookAndFeel(UIManager
						.getSystemLookAndFeelClassName());
			}
		} catch (Exception e) {
		  getLogger().error("Can't set look & feel: ", e);
		}

		if (UIManager.getLookAndFeel().getName().equals("Metal")) {
			UIManager.put(KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY,
					Boolean.TRUE);
			UIManager.put("swing.boldMetal", Boolean.FALSE);
		} else if (UIManager.getLookAndFeel().getName().equals("Windows")) {
			UIManager.put(KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY,
					Boolean.TRUE);
		} else if (UIManager.getLookAndFeel().getName().equals("CDE/Motif")) {
			UIManager.put(KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY,
					Boolean.TRUE);
		}

	}
	
	@SuppressWarnings("unused")
	private static Rectangle getVirtualBounds() {
		Rectangle virtualBounds = new Rectangle();
		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		GraphicsDevice[] gs = ge.getScreenDevices();
		for (int j = 0; j < gs.length; j++) {
			GraphicsDevice gd = gs[j];
			GraphicsConfiguration[] gc = gd.getConfigurations();
			for (int i = 0; i < gc.length; i++) {
				virtualBounds = virtualBounds.union(gc[i].getBounds());
				getLogger().info(Toolkit.getDefaultToolkit().getScreenInsets(gc[i]).toString());
			}
		}
		return virtualBounds;
	}

	private static Insets getScreenInsets() {
		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		GraphicsDevice gd = ge.getDefaultScreenDevice();
		GraphicsConfiguration gc = gd.getDefaultConfiguration();

		return Toolkit.getDefaultToolkit().getScreenInsets(gc);
	}

	public static Dimension getMaximumFrameSize() {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Insets screenInsets = getScreenInsets();
		Dimension ret = new Dimension(screenSize.width - screenInsets.left
				- screenInsets.right, screenSize.height - screenInsets.top
				- screenInsets.bottom);
		return ret;
	}
	
	public static int getCurrentColorDepth() {
		return GraphicsEnvironment.getLocalGraphicsEnvironment()
				.getDefaultScreenDevice().getDisplayMode().getBitDepth();
	}

	public static boolean isDemo() {

		if (KreawiGlobals.isDemo == null) {
			boolean lizenzGueltig = KreawiLicenseHandler.getInstance().hasValidLicence();
      boolean fullVersionAllowed = RegistrationHandler.isFullVersionAllowed();
      if (!lizenzGueltig) {
        getLogger().info("Der Trainer started als Demo, weil keine gültige Lizenz vorliegt!");
        KreawiGlobals.isDemo = true;
      } else if (!fullVersionAllowed) {
        getLogger().info("Der Trainer started als Demo, weil keine gültige Registrierung vorliegt"
            + " oder mehrmals keine Verbindung zum Server aufgebaut werden konnte!");
        KreawiGlobals.isDemo = true;
      } else if (!dataExists()) {
        getLogger().info("Der Trainer started als Demo, weil noch keine Daten installiert wurden!");
        KreawiGlobals.isDemo = true;
      } else {
        KreawiGlobals.isDemo = false;
      }
		}

		return KreawiGlobals.isDemo.booleanValue();
	}
	
	public static boolean dataExists() {
		// Arbeitsverzeichnis ermitteln
		File workDir = new File(System.getProperty("user.dir"));
		File file = new File(new File(workDir, QUESTION_DIR_NAME), QUESTION_FILE_NAME);
		
		if (!file.exists()) {
		  return false;
		}

		try {
      return verifyFileIntegrity(file, QUESTION_FILE_SIZE, QUESTION_FILE_HASH);
		} catch (NoSuchAlgorithmException e) {
		  getLogger().error("Application will run in demo mode, as file integrity could not be verfified!", e);
		} catch (IOException e) {
		  getLogger().error("Application will run in demo mode, as file integrity could not be verfified!", e);
		}
		return false;
	}
	
	public static boolean verifyFileIntegrity(File file, long size,
			String md5Hash) throws IOException, NoSuchAlgorithmException {
	  
	  if (!file.exists()) {
	    throw new IOException("Datei '" + file.getAbsolutePath() + "' nicht gefunden!");
	  }

		if (KreawiGlobals.isDevMode()) {
		  getLogger().info("File integrity check:");
		  getLogger().info("File name: " + file.getAbsolutePath());
		  getLogger().info("File hash: " + Util.md5FileHash(file));
		  getLogger().info("File size: " + file.length());
		  getLogger().info("Expected file hash: " + md5Hash);
		  getLogger().info("Expected file size: " + size);
		}

		return Util.verifyFileIntegrity(file, size, md5Hash);

	}

	public static boolean isDevMode() {
		return isDevMode;
	}

	public static void setDevMode(boolean isDevMode) {
		KreawiGlobals.isDevMode = isDevMode;
	}
	
	public static boolean isBetaMode() {
	  return isBetaMode;
	}

  public static void setBetaMode(boolean isDevMode) {
    KreawiGlobals.isBetaMode = isDevMode;
  }
  
  public static boolean isSandboxEnabled() {
    final String enabled = System.getProperty("SandboxEnabled");
    return "false".equals(enabled);   // Falsch rum???
  }
  
  private static void writeGlobalSetting(final String filename, final String value) {
    final File dir = new File(getApplicationDirectory(), GLOBAL_SETTINGS_DIR);
    dir.mkdirs();
    final Path path = new File(dir, filename).toPath();
    try {
      Files.write(path, Arrays.asList(value), Charset.forName("UTF-8"));
    } catch (IOException e) {
      getLogger().error("Error writing global setting! (filename: " + filename + ", value: " + value + ")", e);
    }
  }
  
  private static String readGlobalSetting(final String filename) {
    final File dir = new File(getApplicationDirectory(), GLOBAL_SETTINGS_DIR);
    final File file = new File(dir, filename);
    if (!file.exists()) {
      return null;
    }
    try {
      return String.join("|", Files.readAllLines(file.toPath(), Charset.forName("UTF-8")));
    } catch (IOException e) {
      getLogger().error("Error reading global setting! (filename: " + filename + ")", e);
      return null;
    }
  }
  
  public static String getInstallDir() {
    return readGlobalSetting(GLOBAL_SETTING_INSTALL_DIR);
  }
  
  public static void setInstallDir(final String dir) {
    writeGlobalSetting(GLOBAL_SETTING_INSTALL_DIR, dir);
  }
  
  public static long getAverageStartupTimeMillis() {
    try {
      final String value = readGlobalSetting(GLOBAL_SETTING_AVERAGE_STARTUP_TIME);
      if (value == null) {
        return DEFAULT_AVERAGE_STARTUP_TIME;
      }
      return Long.parseLong(value);
    } catch (NumberFormatException e) {
      getLogger().error("Error reading average startup time!", e);
      return DEFAULT_AVERAGE_STARTUP_TIME;
    }
  }
  
  public static void updateAverageStartupTimeMillis(final long lastStartupTime) {
    final long oldAvgStartupTime = getAverageStartupTimeMillis();
    final long newAvgStartupTime;
    if (oldAvgStartupTime != DEFAULT_AVERAGE_STARTUP_TIME) {
      newAvgStartupTime = (long) ((double) oldAvgStartupTime + 0.1 * ((double) (lastStartupTime - oldAvgStartupTime)));
    } else {
      newAvgStartupTime = lastStartupTime;
    }

    writeGlobalSetting(GLOBAL_SETTING_AVERAGE_STARTUP_TIME, Long.toString(newAvgStartupTime));
  }
  
  public static boolean isFullVersionUpdate() {
    return (readGlobalSetting(GLOBAL_SETTING_IS_FULL_VERSION_UPDATE) != null);
  }
  
  public static void setFullVersionUpdate(final boolean isFullVersionUpdate) {
    if (isFullVersionUpdate) {
      writeGlobalSetting(GLOBAL_SETTING_IS_FULL_VERSION_UPDATE, "");
    } else {
      final File dir = new File(getApplicationDirectory(), GLOBAL_SETTINGS_DIR);
      final File file = new File(dir, GLOBAL_SETTING_IS_FULL_VERSION_UPDATE);
      file.delete();
    }
  }
  
  public static void openLogDir() throws IOException {
    final File logDir = new File(getApplicationDirectory(), "logs");
    Desktop.getDesktop().open(logDir);
  }
  
  // Don't init on variable decaration because this class is used before reading the logger config!
  private static Logger getLogger() {
    if (logger == null) {
      logger = LoggerFactory.getLogger(KreawiGlobals.class);
    }
    return logger;
  }

}
