package de.kreawi.trainer.db;

import java.util.function.Supplier;

import javax.sql.DataSource;

import org.javalite.activejdbc.Base;

public abstract class DbAccessModel {
  
  private final DataSource dataSource;
  
  public DbAccessModel(final DataSource dataSource) {
    this.dataSource = dataSource;
  }

  protected void runWithDbConn(final Runnable runnable) {
    callWithDbConn(() -> {
      runnable.run();
      return null;
    });
  }

  protected <T> T callWithDbConn(final Supplier<T> callable) {
    openDbConn();
    try {
      return callable.get();
    } finally {
      closeDbConn();
    }
  }

  private void openDbConn() {
    if (!Base.hasConnection()) {
      Base.open(this.dataSource);
    }
  }

  private void closeDbConn() {
    if (Base.hasConnection()) {
      Base.close();
    }
  }
  
}
