package de.kreawi.trainer.db;

import org.javalite.activejdbc.Base;
import org.javalite.activejdbc.Model;

public class QuestionAttributes extends Model {
  
  public static final String DB_ID = "id";
  public static final String DB_QUESTION_ID = "question_id";
  public static final String DB_CARD_BOX_LEVEL = "card_box_level";
  public static final String DB_CARD_BOX_TIMESTAMP = "card_box_timestamp";
  public static final String DB_ICU = "icu";
  public static final String DB_NOTE = "note";
  
  public static void createTableIfNotExists() {
    final String tableName = "question_attributes";
    final String createQuery = "CREATE TABLE IF NOT EXISTS " + tableName + " (" +
        DB_ID + " INT AUTO_INCREMENT PRIMARY KEY, " +  
        DB_QUESTION_ID + " INTEGER NULL, " +  
        DB_CARD_BOX_LEVEL + " TINYINT NOT NULL DEFAULT " + QuestionAttributesModel.CARD_BOX_LEVEL_UNDEFINED + ", " +  
        DB_CARD_BOX_TIMESTAMP + " BIGINT NOT NULL DEFAULT -1, " +  
        DB_ICU + " BOOLEAN NOT NULL DEFAULT False, " +
        DB_NOTE + " VARCHAR NULL" +
        ");";
    Base.exec(createQuery);
    
    createIndexIfNotExists("question_id_index", tableName, DB_QUESTION_ID);
    createIndexIfNotExists("card_box_level_index", tableName, DB_CARD_BOX_LEVEL);
    createIndexIfNotExists("card_box_timestamp_index", tableName, DB_CARD_BOX_TIMESTAMP);
    createIndexIfNotExists("icu_index", tableName, DB_ICU);
    
  }
  
  private static void createIndexIfNotExists(final String indexName, final String tableName, final String indexField) {
    final String indexQuery = "CREATE INDEX IF NOT EXISTS " + indexName + " ON " + tableName + "(" + indexField + ");";
    Base.exec(indexQuery);
  }

  public int getQuestionId() {
    return getInteger(DB_QUESTION_ID).intValue();
  }
  
  // No setter for id
  
  public int getCardBoxLevel() {
    return getInteger(DB_CARD_BOX_LEVEL).intValue();
  }
  
  public void setCardBoxLevel(final int level) {
    set(DB_CARD_BOX_LEVEL, level);
  }

  public long getCardBoxTimestamp() {
    return getLong(DB_CARD_BOX_TIMESTAMP).longValue();
  }
  
  public void setCardBoxTimestamp(final long timestamp) {
    set(DB_CARD_BOX_TIMESTAMP, timestamp);
  }
  
  public boolean isIcu() {
    return getBoolean(DB_ICU);
  }
  
  public void setIcu(final boolean icu) {
    set(DB_ICU, icu);
  }
  
  public String getNote() {
    return getString(DB_NOTE);
  }
  
  public void setNote(final String note) {
    set(DB_NOTE, note);
  }
  
  @Override
  protected void afterSave() {
    super.afterSave();
    // refresh to retrieve default values of columns
    this.refresh();
  }
  
}
