package de.kreawi.trainer.dialogs;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Paint;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.NumberFormat;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.Timer;
import javax.swing.UIManager;

import org.jCharts.chartData.PieChartDataSet;
import org.jCharts.nonAxisChart.PieChart2D;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.properties.PieChart2DProperties;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.ButtonBarBuilder;
import com.jgoodies.forms.builder.ButtonStackBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.uif_lite.component.Factory;
import com.jquizengine.questions.Question;
import com.jquizengine.quiz.Quiz;
import com.jquizengine.quiz.Response;

import de.kreawi.trainer.KreawiGlobals;
import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerMain;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.dialogs.quiz.QuizTitleClock;
import de.kreawi.trainer.util.TrainerHtmlUtil;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.util.JScrollablePanel;
import de.martinknauer.util.Util;

@SuppressWarnings("serial")
public class EvaluationDialog extends KreawiDialogPanel implements Observer {

	public static final String ICON_CORRECT_PATH = KreawiGlobals.APP_IMAGES_DIR_NAME + "/status_display_ok.png";

	public static final String ICON_WRONG_PATH = KreawiGlobals.APP_IMAGES_DIR_NAME + "/status_display_false.png";
	
	private static final Dimension maxSize = new Dimension((int) (Util
			.isLargeFontSize() ? TrainerFrame.FRAME_SIZE.width * 0.9
			: (TrainerFrame.FRAME_SIZE.width * 0.7)),
			TrainerFrame.FRAME_SIZE.height - 90);
	
	private static final Dimension minSize = new Dimension((int) (Util
			.isLargeFontSize() ? TrainerFrame.FRAME_SIZE.width * 0.9
			: (TrainerFrame.FRAME_SIZE.width * 0.7)),
			TrainerFrame.FRAME_SIZE.height - 260);

	private Map<Actions, Action> actionMap;

	private Quiz selectedQuestions;

	private JList<QuestionTuple> list;

	private JLabel evaluationLabel;

	private PieChart2D pieChart2D;

	private double[] data;

	private JPopupMenu popup;

	private TrainerModel model;

	private Timer clock;

	private AbstractAction backToQuizAction;

  private static final Logger logger = LoggerFactory.getLogger(EvaluationDialog.class); 

	/**
	 * Builds the dialog.
	 */
	public EvaluationDialog(Window parentComp, TrainerModel model,
			Map<Actions, Action> actionMap) {

		this.model = model;
		this.selectedQuestions = this.model.getSelectedQuestions();
		this.selectedQuestions.addObserver(this);
		this.actionMap = actionMap;

		setLayout(new GridBagLayout());
		setOpaque(false);

		final JPanel panel = new JPanel(new BorderLayout());
		panel.setOpaque(false);
		panel.setBorder(Borders.DIALOG_BORDER);
		
		Dimension maxFrameSize = Util.getMaximumFrameSize(parentComp
				.getGraphicsConfiguration());

		if (maxFrameSize.height > TrainerFrame.FRAME_SIZE.height) {
			// general case
			panel.setPreferredSize(EvaluationDialog.maxSize);
		} else {
			// netbook case
			panel.setPreferredSize(EvaluationDialog.minSize);
		}
		
		panel.setMaximumSize(EvaluationDialog.maxSize);

		panel.add(buildStaticVerticalSplit(buildStaticHorizontalSplit(
				buildStaticVerticalSplit(buildQuestionListPanel(),
						buildUtilityButtonsPanel(), 70, 1d), buildStatsPanel(),
				40, 0.2), buildNavigationButtonsPanel(), 90, 1d));

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;

		add(panel, gbc);

		addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent e) {
				adjustPreferredSize(panel);
			}

			public void componentResized(ComponentEvent e) {
				adjustPreferredSize(panel);
			}
		});
	}
	
	private Dimension calculatePreferedSize() {

		Dimension visibleSpace = getVisibleRect().getSize();

		if (visibleSpace.height < minSize.height) {
			return new Dimension(minSize);
		}

		if (maxSize.height > visibleSpace.height) {
			return new Dimension(maxSize.width, visibleSpace.height);
		}

		return new Dimension(maxSize);
	}

	private void adjustPreferredSize(JPanel panel) {
		Dimension newPreferredSize = calculatePreferedSize();

		if (!newPreferredSize.equals(getPreferredSize())) {
			panel.setPreferredSize(newPreferredSize);
		}

		revalidate();
		repaint();
	}

	private JComponent buildStatsPanel() {
		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("EVALUATION_TITLE_STATS")
				+ " - "
				+ this.selectedQuestions.getStopwatch()
						.getOverallTimeTakenString(), new BorderLayout());

		this.clock = new QuizTitleClock(500, this.selectedQuestions, kif
				.getTitleLabel(), TrainerTextResources
				.getString("EVALUATION_TITLE_STATS")
				+ " - ", "", this.model.getConfig()
				.getLong("questionTimeLimit"), selectedQuestions.isExamMode());
		clock.setInitialDelay(0);
		clock.start();

		final JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());
		gradientPanel.setPreferredSize(new Dimension(360, 500));

		final JPanel evaluationPanel = new JScrollablePanel() {
			@Override
			public boolean getScrollableTracksViewportWidth() {
				return true;
			}
			
			public boolean getScrollableTracksViewportHeight() {
				return gradientPanel.getSize().height >= getMinimumSize().height;
			}
		};
		evaluationPanel.setLayout(new BoxLayout(evaluationPanel,
				BoxLayout.PAGE_AXIS));
		evaluationPanel.setBorder(Borders.DIALOG_BORDER);
		evaluationPanel.setOpaque(false);
		
		JPanel chartPanel = new JPanel() {
			public void paint(Graphics g) {
				super.paint(g);
				try {
					pieChart2D.setGraphics2D((Graphics2D) g);
					pieChart2D.render();
				} catch (Exception e) {
          logger.error("Problem encountered when trying to render pie chart!", e);
				}
			}
		};

		this.data = new double[3];
		updatePieData();

		String[] labels = {
				TrainerTextResources.getString("EVALUATION_PIE_LEGEND_CORRECT"),
				TrainerTextResources.getString("EVALUATION_PIE_LEGEND_WRONG"),
				TrainerTextResources.getString("EVALUATION_PIE_LEGEND_UNANSWERED") };

		Paint[] paints = {
				new GradientPaint(0f, 0f, new Color(0, 165, 55), 200f, 0f,
						new Color(0, 75, 55)),
				new GradientPaint(0f, 0f, new Color(165, 0, 0), 200f, 0f,
						new Color(75, 0, 0)),
				new GradientPaint(0f, 0f, new Color(75, 75, 75, 255), 200f, 0f,
						new Color(75, 75, 75, 0)) };

		PieChart2DProperties pieChart2DProperties = new PieChart2DProperties();
		pieChart2DProperties.setBorderStroke(new BasicStroke(0.5f));
		pieChart2DProperties.setZeroDegreeOffset(180f);

		try {
			PieChartDataSet pieChartDataSet = new PieChartDataSet(null, data,
					labels, paints, pieChart2DProperties);

			ChartProperties cp = new ChartProperties();
			cp.setBackgroundPaint(new Color(0, 0, 0, 0));

			LegendProperties lp = new LegendProperties();
			lp.setBorderStroke(null);
			Font font = new JLabel().getFont();
			lp.setFont(new Font(font.getName(), font.getStyle(), 11));

			this.pieChart2D = new PieChart2D(pieChartDataSet, lp, cp, 200, 200);

		} catch (Exception e) {
		  logger.error("Problem encountered when setting up pie chart!", e);
		}

		chartPanel.setOpaque(false);
		chartPanel.setPreferredSize(new Dimension(this.pieChart2D
				.getImageWidth(), this.pieChart2D.getImageHeight()));
		chartPanel.setMinimumSize(chartPanel.getPreferredSize());

		JPanel outerChartPanel = new JPanel(new GridBagLayout());
		outerChartPanel.setBorder(BorderFactory.createEmptyBorder());
		outerChartPanel.setOpaque(false);

		outerChartPanel.add(chartPanel);

		evaluationPanel.add(outerChartPanel);

		this.evaluationLabel = new JLabel(generateEvaluationHtml(), JLabel.CENTER);

		JPanel outerLabelPanel = new JPanel(new GridBagLayout());
		outerLabelPanel.setBorder(BorderFactory.createEmptyBorder());
		outerLabelPanel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridy = 0;
		gbc.gridx = 1;
		gbc.weightx = 1;
		gbc.weighty = 1;
		
		outerLabelPanel.add(this.evaluationLabel, gbc);

		evaluationPanel.add(outerLabelPanel);

		JScrollPane scrollPane = Factory
				.createStrippedScrollPane(evaluationPanel);
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);
		

		gradientPanel.add(scrollPane);

		return kif;
	}

	private String generateEvaluationHtml() {

		int all = this.selectedQuestions.getQuestionCount();
		int answered = this.selectedQuestions.getAnsweredQuestionCount();
		int correct = this.selectedQuestions.getCorrectAnswersCount();
		int incorrect = this.selectedQuestions.getIncorrectAnswersCount();

		double percentCorrectAnswered = answered > 0 ? (double) correct
				/ (double) answered : 0d;
		double percentAllAnswered = all > 0 ? (double) correct / (double) all
				: 0;

		String timePassed = this.selectedQuestions.getStopwatch()
				.getOverallAnsweredTimeTakenMinutesString();

		String conclusion = "";

		if (percentAllAnswered >= 0.75
				&& this.selectedQuestions.isQuizTimeLimitExceeded(this.model
						.getConfig().getLong("questionTimeLimit"))) {
			conclusion = TrainerTextResources
					.getString("QUIZ_EVALUATION_CONCLUSION_TEXT_FOR_MC>75_SLOW");
		} else if (percentAllAnswered >= 0.75) {
			conclusion = TrainerTextResources
					.getString("QUIZ_EVALUATION_CONCLUSION_TEXT_FOR_MC>75");
		} else if (percentAllAnswered >= 0.50 && answered == all) {
			conclusion = TrainerTextResources
					.getString("QUIZ_EVALUATION_CONCLUSION_TEXT_FOR_MC>50");
		} else if (answered == all) {
			conclusion = TrainerTextResources
					.getString("QUIZ_EVALUATION_CONCLUSION_TEXT_FOR_MC<50");
		}

		String ret = "";

		String[] subsitutes = {
				answered + "",
				all + "",
				correct + "",
				incorrect + "",
				NumberFormat.getPercentInstance()
						.format(percentCorrectAnswered),
				NumberFormat.getPercentInstance().format(percentAllAnswered),
				timePassed, conclusion };

		ret = TrainerTextResources.getStringWithSubstitution(
				"EVALUATION_TEXT_FOR_MC", subsitutes);

		return ret;
	}

	private JComponent buildQuestionListPanel() {
		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("EVALUATION_TITLE_QUESTIONS"), new BorderLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());
		gradientPanel.setPreferredSize(new Dimension(240, 500));

		this.popup = new JPopupMenu();
		JMenuItem item = new JMenuItem(TrainerTextResources
				.getString("EVALUATION_TEXT_JUMP_TO_QUESTION"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jumpToQuestion(list.getSelectedIndex());
			}
		});
		popup.add(item);
		
		item = new JMenuItem(TrainerTextResources
				.getString("EVALUATION_TEXT_QUESTION_INTO_ICU"));
		item.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addQuestionToIcu(list.getSelectedIndex());
			}
		});
		popup.add(item);
		
		Vector<QuestionTuple> questionTuples = new Vector<QuestionTuple>();

		for (Question question : this.selectedQuestions.getQuestions()) {
			questionTuples.add(new QuestionTuple(question,
					this.selectedQuestions.getResponse(question)));
		}

		this.list = new JList<QuestionTuple>(questionTuples) {
			@Override
			public boolean getScrollableTracksViewportWidth() {
				return true;
			}
		};
		this.list.setCellRenderer(new QuestionReplyCellRenderer());
		this.list.addMouseListener(new MouseAdapter() {

			public void mouseClicked(MouseEvent e) {
				mouseEventPerfomed(e);
			}

			public void mousePressed(MouseEvent e) {
				mouseEventPerfomed(e);
			}

			public void mouseReleased(MouseEvent e) {
				mouseEventPerfomed(e);
			}

		});

		if (Boolean.TRUE.equals(UIManager.getDefaults().get(
				KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY))) {
			this.list.setOpaque(false);
		}

		this.list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		JScrollPane scrollPane = Factory.createStrippedScrollPane((this.list));
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

		gradientPanel.add(scrollPane);

		setFocusJComponent(this.list);

		return kif;
	}

	private KreawiInternalFrame buildUtilityButtonsPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("EVALUATION_TITLE_INCORRECT"),
				new GridLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setLayout(new GridBagLayout());
		gradientPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 11, 10));

		JButton repeatIncorrectButton = new JButton(this.actionMap
				.get(Actions.REPEAT_INCORRECT));
		repeatIncorrectButton.setToolTipText(TrainerTextResources
				.getString("EVALUATION_BUTTON_REPEAT_INCORRECT_TOOL_TIP"));

		JButton intensiveCareButton = new JButton(this.actionMap
				.get(Actions.INCORRECT_QUESTIONS_INTO_ICU));
		intensiveCareButton.setToolTipText(TrainerTextResources
				.getString("EVALUATION_BUTTON_INCORRECT_TO_ICU_TOOL_TIP"));
		intensiveCareButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				list.revalidate();
				list.repaint();
			}
		});

		updateActions();
		
		ButtonStackBuilder builder = new ButtonStackBuilder();
		
		builder.addGlue();
		builder.addGridded(repeatIncorrectButton);
		builder.addRelatedGap();
		builder.addGridded(intensiveCareButton);
		builder.addGlue();
		
		JPanel buttonsPanel = builder.getPanel();
		buttonsPanel.setOpaque(false);

		gradientPanel.add(buttonsPanel);
		
		gradientPanel.setMinimumSize(gradientPanel.getPreferredSize());

		return kif;
	}
	
	private KreawiInternalFrame buildNavigationButtonsPanel() {

		KreawiInternalFrame kif = new KreawiInternalFrame(null,
				new GridBagLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 11, 10));
		gradientPanel.setLayout(new GridBagLayout());
			
		final Action tempAction = this.actionMap.get(Actions.BACK_TO_QUIZ);

		this.backToQuizAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				tempAction.actionPerformed(e);
			}
		};

		if (!this.selectedQuestions.hasUnanswered()) {
			backToQuizAction.putValue(Action.NAME, TrainerTextResources
					.getString("EVALUATION_BUTTON_BACK_TO_QUIZ_ALL_ANSWERED"));
		} else {
			backToQuizAction.putValue(Action.NAME, TrainerTextResources
					.getString("EVALUATION_BUTTON_BACK_TO_QUIZ"));
		}

		JButton backQuizButton = new JButton(backToQuizAction);

		JButton backMainButton = new JButton(this.actionMap
				.get(Actions.BACK_TO_MAIN_DIALOG));

		updateActions();
		
		ButtonBarBuilder builder = new ButtonBarBuilder();
		
		if (builder.isLeftToRightButtonOrder()) {
			builder.addGlue();
			builder.addGridded(backQuizButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(backMainButton);
			builder.addGlue();
		} else {
			builder.addGlue();
			builder.addGridded(backMainButton);
			builder.addGlue();
			builder.addGlue();
			builder.addGridded(backQuizButton);
			builder.addGlue();
		}
		
		JPanel buttonsPanel = builder.getPanel();
		buttonsPanel.setOpaque(false);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		
		gradientPanel.add(buttonsPanel, gbc);
		
		gradientPanel.setMinimumSize(gradientPanel.getPreferredSize());

		return kif;
	}

	private class QuestionReplyCellRenderer implements ListCellRenderer<QuestionTuple> {

		public Component getListCellRendererComponent(JList<? extends QuestionTuple> list, QuestionTuple value,
				int index, boolean isSelected, boolean cellHasFocus) {
			
			QuestionTuple tuple = (QuestionTuple) value;
			
			Question question = tuple.question;
			Response reply = tuple.reply;

			JPanel currentPanel = new JPanel(new GridLayout(1, 4));
			currentPanel.setBorder(BorderFactory.createEmptyBorder(2, 6, 2, 6));

			JLabel textLabel = new JLabel(TrainerTextResources
					.getString("EVALUATION_TEXT_QUESTION")
					+ " " + (index + 1));
			textLabel.setOpaque(false);

			JLabel iconLabel;
			textLabel.setOpaque(false);

			if (reply.isCorrect()) {
				iconLabel = new JLabel(new ImageIcon(TrainerMain.class.getResource(ICON_CORRECT_PATH)));
			} else if (reply.isAnswered()) {
				iconLabel = new JLabel(new ImageIcon(TrainerMain.class.getResource(ICON_WRONG_PATH)));
			} else {
				iconLabel = new JLabel("-", JLabel.CENTER);
				iconLabel.setBorder(BorderFactory.createEmptyBorder(0, 2, 0, 0));
			}
			
			JLabel icuLabel = new JLabel(
					model.isInIntensiveCare(question) ? "ICU" : "",
					JLabel.CENTER);
			
			JLabel timeTakenLabel = new JLabel(
					reply.getTimeTaken() > 999 || reply.isAnswered()
							|| selectedQuestions.isExamMode() ? Util
							.millisecondsTommss(reply.getTimeTaken()) : "",
					JLabel.RIGHT);

			currentPanel.add(textLabel);
			currentPanel.add(iconLabel);
			currentPanel.add(icuLabel);
			currentPanel.add(timeTakenLabel);

			currentPanel.setBackground(UIManager
					.getColor("List.selectionBackground"));
			currentPanel.setOpaque(isSelected);
			
			if (isSelected) {
				textLabel.setForeground(UIManager
					.getColor("List.selectionForeground"));
				iconLabel.setForeground(UIManager
						.getColor("List.selectionForeground"));
				icuLabel.setForeground(UIManager
						.getColor("List.selectionForeground"));
				timeTakenLabel.setForeground(UIManager
						.getColor("List.selectionForeground"));
				
			}

      final boolean justify = actionMap.get(Actions.JUSTIFY_TEXT).getValue(TrainerFrame.ACTION_SELECTED_KEY).equals(Boolean.TRUE);
      final String css = TrainerHtmlUtil.getDefaultCss(justify);
      currentPanel.setToolTipText(TrainerHtmlUtil.wrapInHtml(question.getText(), css));

			return currentPanel;
		}

	}

	private void updatePieData() {
		this.data[0] = this.selectedQuestions.getCorrectAnswersCount();
		this.data[1] = this.selectedQuestions.getIncorrectAnswersCount();
		this.data[2] = this.selectedQuestions.getUnansweredQuestionCount();
	}
	
	private void updateActions() {
		this.actionMap.get(Actions.REPEAT_INCORRECT).setEnabled(
				this.selectedQuestions.getIncorrectAnswersCount() > 0);
		this.actionMap.get(Actions.INCORRECT_QUESTIONS_INTO_ICU).setEnabled(
				this.selectedQuestions.getIncorrectAnswersCount() > 0);
	}

	public void update(Observable o, Object arg) {
		this.evaluationLabel.setText(generateEvaluationHtml());
		updatePieData();
		updateActions();
		if (!this.selectedQuestions.hasUnanswered()) {
			backToQuizAction.putValue(Action.NAME, TrainerTextResources
					.getString("EVALUATION_BUTTON_BACK_TO_QUIZ_ALL_ANSWERED"));
		}
	}

	private void jumpToQuestion(int index) {
		this.selectedQuestions.setIndex(index);
		new JButton(this.actionMap
				.get(TrainerFrame.Actions.NEW_QUESTION_SELECTED)).doClick();
		new JButton(this.actionMap.get(TrainerFrame.Actions.BACK_TO_QUIZ))
				.doClick();
	}
	
	private void addQuestionToIcu(int index) {
		Question question = this.selectedQuestions.getQuestion(index);
		if (this.model.isInIntensiveCare(question)) {
			this.model.removeQuestionFromIcu(question);
		} else {
			this.model.addQuestionToIcu(question);
		}
	}

	private void mouseEventPerfomed(MouseEvent e) {

		if (e.getClickCount() == 2) {
			int index = list.locationToIndex(e.getPoint());
			jumpToQuestion(index);
		} else if (e.isPopupTrigger()) {
			int index = list.locationToIndex(e.getPoint());
			list.setSelectedIndex(index);
			popup.show(e.getComponent(), e.getX(), e.getY());
		}

	}
	
	public void closeDialog() {
		this.clock.stop();
		this.clock = null;
	}

	private static class QuestionTuple {
		private Question question;
		private Response reply;

		private QuestionTuple(Question question, Response reply) {
			this.question = question;
			this.reply = reply;
		}
	}
}
