package de.kreawi.trainer.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.UIManager;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.kreawi.trainer.TrainerFrame;
import de.martinknauer.awt.Gradient.Direction;
import de.martinknauer.swing.GradientPanel;

@SuppressWarnings("serial")
public class KreawiInternalFrame extends SimpleInternalFrame {

	public final static String USE_GRADIENT_BACKGROUND_KEY = "useGradientBackground";
	
	public final static String USE_LOW_COLOR_DEPTH_BACKGROUND_KEY = "useLowColorDepthBackground";

  private static final Logger logger = LoggerFactory.getLogger(TrainerFrame.class); 

	public KreawiInternalFrame(String title, LayoutManager lm) {
		this(title, lm, null);
	}

	public KreawiInternalFrame(String title, LayoutManager lm,
			JComponent rightHeaderComponent) {
		this(title, lm, rightHeaderComponent, Boolean.TRUE.equals(UIManager
				.getDefaults().get(USE_GRADIENT_BACKGROUND_KEY)), Boolean.TRUE
				.equals(UIManager.getDefaults().get(
						USE_LOW_COLOR_DEPTH_BACKGROUND_KEY)));
	}

	public KreawiInternalFrame(String title, LayoutManager lm,
			JComponent rightHeaderComponent, boolean useGradient,
			boolean useLowColorDepthBackground) {
		this(title, lm, rightHeaderComponent, useGradient,
				useLowColorDepthBackground, true);
	}

	public KreawiInternalFrame(String title, LayoutManager lm,
			JComponent rightHeaderComponent, boolean useGradient,
			boolean useLowColorDepthBackground, boolean drawShadow) {
		super(null, title, rightHeaderComponent, null, drawShadow);

		if (title == null) {
			/* remove header bar if a null title is passed */
			setLayout(new BorderLayout());
		}

		if (useGradient) {
			
			if (useLowColorDepthBackground) {
				JPanel panel = new JPanel(lm);
				
				Color bg = panel.getBackground();
				panel.setBackground(new Color(bg.getRed() + 3, bg.getGreen() + 3,
						bg.getBlue() + 3));
	
				GradientPanel gradientPanel = new GradientPanel(lm, panel.getBackground(),
						Direction.VERTICAL, false, 2);
							
				setContent(gradientPanel);
			} else {
				GradientPanel gradientPanel = new GradientPanel(lm, Color.WHITE,
						Direction.DIAGONAL);
				setContent(gradientPanel);
			}
			
		} else {
			JPanel panel = new JPanel(lm);
			
			if (useLowColorDepthBackground) {
				panel.setBackground(panel.getBackground().brighter());
			} else {
			  panel.setBackground(Color.WHITE);
			}
			
			setContent(panel);
		}

	}

	public void setContent(Component newContent) {

		if (!(newContent instanceof JPanel)) {
			throw new IllegalArgumentException(
					"Content of KreawiInternalFrame must be of type JPanel!");
		}

		super.setContent(newContent);
	}

	public JPanel getContent() {
		return (JPanel) super.getContent();
	}
	
	public JLabel getTitleLabel() {
		return this.titleLabel;
	}

	public static JToolBar createLabeledIconButtonToolBar(Icon icon,
			String text, String toolTip, ActionListener actionListener) {
		return createLabeledIconButtonToolBar(icon, null, null, text, toolTip,
				actionListener);
	}
	
	public static JToolBar createLabeledIconButtonToolBar(Icon icon,
			Icon rolloverIcon, Icon pressedIcon, String text,
			String toolTip, ActionListener actionListener) {
		JToolBar toolBar = new JToolBar() {

			@Override
			public void setEnabled(boolean enabled) {
				super.setEnabled(enabled);
				for (Component comp : getComponents()) {
					comp.setEnabled(enabled);
				}
			}
			
		};
		
		JButton switchButton = createIconButton(icon, rolloverIcon,
				pressedIcon, toolTip, actionListener);

		if (text != null && !text.equals("")) {
			JLabel label = new JLabel(text);
			label.setBorder(BorderFactory.createEmptyBorder(1, 0, 0, 2));
			toolBar.add(label);
		}
		toolBar.add(switchButton);
		toolBar.setOpaque(false);
		toolBar.setBorder(BorderFactory.createEmptyBorder());
		toolBar.setFloatable(false);
		toolBar.setRollover(true);
		
		JPanel gluePanel = new JPanel();
		gluePanel.setBorder(BorderFactory.createEmptyBorder());
		gluePanel.setPreferredSize(new Dimension(2, 2));
		gluePanel.setMaximumSize(gluePanel.getPreferredSize());
		gluePanel.setOpaque(false);
		toolBar.add(gluePanel);
		
//		toolBar.putClientProperty("JToolBar.isRollover", Boolean.TRUE);
		
		return toolBar;
	}
	
	public static JButton createIconButton(Icon icon,
			Icon rolloverIcon, Icon pressedIcon,
			String toolTip, ActionListener actionListener) {
		
		final JButton switchButton = new JButton(icon);
		
		if (rolloverIcon != null) {
			switchButton.setRolloverIcon(rolloverIcon);
			switchButton.setRolloverEnabled(true);
			switchButton.setBorderPainted(false);
		}
		
		if (pressedIcon != null) {
			switchButton.setPressedIcon(pressedIcon);
		}

		switchButton.setOpaque(false);
		if (toolTip != null && !toolTip.equals("")) {
			switchButton.setToolTipText(toolTip);
		}
		switchButton.addActionListener(actionListener);

		switchButton.setFocusPainted(false);
		switchButton.setContentAreaFilled(false);

		return switchButton;
	}
	
}
