package de.kreawi.trainer.dialogs;

import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Pane;
import javafx.scene.text.Text;

public class KreawiInternalFrameFx extends BorderPane {

  public KreawiInternalFrameFx(final Pane content) {
    this(null, content);
  }
 
  public KreawiInternalFrameFx(final String titleLeft, final Pane content) {
    this(titleLeft, null, content);
  }
    
  public KreawiInternalFrameFx(final String titleLeft, final Node titleRight, final Pane content) {
    super(content);
    
    getStylesheets().add("/de/kreawi/trainer/dialogs/kreawi_internal_frame.css");
    // dropshadow for the whole thing
    getStyleClass().add("kif");

    // linear gradient as background for the content
    content.getStyleClass().add("kif_content");
    
    if ((titleLeft != null) || (titleRight != null )) {
      final BorderPane titleContainer = new BorderPane();
      titleContainer.getStyleClass().add("kif_title_bar");
      
      if (titleLeft != null) {
        final Text titleLabel = new Text(titleLeft);
        BorderPane.setAlignment(titleLabel, Pos.CENTER_LEFT);
        titleContainer.setLeft(titleLabel);
      }
      
      if (titleRight != null) {
        BorderPane.setAlignment(titleRight, Pos.CENTER_RIGHT);
        titleContainer.setRight(titleRight);
      }
      
      setTop(titleContainer);
      content.getStyleClass().add("kif_content_with_title");
    } else {
      content.getStyleClass().add("kif_content_no_title");
    }

  }
  
  public static Button createRightButton(final Runnable action) {
    final String cssClass = "right-button";
    return createTitleButton(action, cssClass);
  }

  public static Button createLeftButton(final Runnable action) {
    final String cssClass = "left-button";
    return createTitleButton(action, cssClass);
  }

  protected static Button createTitleButton(final Runnable action, final String cssClass) {
    final Button button = new Button();
    button.setOnAction(new EventHandler<ActionEvent>() {
      @Override
      public void handle(ActionEvent event) {
        action.run();
      }
    });
    button.getStylesheets().add("/de/kreawi/trainer/dialogs/kreawi_internal_frame.css");
    button.getStyleClass().addAll("title-button", cssClass);
    return button;
  }
  
}
