package de.kreawi.trainer.dialogs;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.component.Factory;

import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.util.TrainerTextResources;
import de.martinknauer.util.config.AbstractConfigComponent;
import de.martinknauer.util.config.Config;
import de.martinknauer.util.config.ConfigComponentHandler;

@SuppressWarnings("serial")
public class PreferencesDialog extends KreawiDialogPanel {

	private Config config;

	private Config settings;

	private ConfigComponentHandler configCompHandler;

  private static final Logger logger = LoggerFactory.getLogger(PreferencesDialog.class); 

	/**
	 * Builds the dialog.
	 */
  public PreferencesDialog(Config config, Config settings, ConfigComponentHandler configCompHandler) {
    
		this.config = config;
		this.settings = settings;

		this.configCompHandler = configCompHandler;

		setLayout(new GridBagLayout());
		setOpaque(false);
		setBorder(Borders.DIALOG_BORDER);

		JPanel panel = new JPanel(new GridBagLayout());

		panel.setBorder(BorderFactory.createEmptyBorder());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.weightx = 1;
		gbc.weighty = 1;
		
		final int width = TrainerFrame.MEDIUM_DIALOG_SIZE.width - 200;
		final int height = TrainerFrame.MEDIUM_DIALOG_SIZE.height - 10;
    
		panel.setPreferredSize(new Dimension(width, height));
		panel.setMinimumSize(getPreferredSize());
		panel.setMaximumSize(getPreferredSize());
    final JPanel prefPanel = buildSettingsPanel();
    final JScrollPane scrollPane = Factory.createStrippedScrollPane(prefPanel);
    scrollPane.setBorder(BorderFactory.createEmptyBorder());
    
    scrollPane.setPreferredSize(new Dimension(width, height));
    scrollPane.setMinimumSize(getPreferredSize());
    scrollPane.setMaximumSize(getPreferredSize());
    
		panel.add(scrollPane, gbc);
		add(panel);
	}

	protected JPanel buildSettingsPanel() {

		AbstractConfigComponent<JComboBox<String>> questionCommentBox = new AbstractConfigComponent<JComboBox<String>>(
				buildFontSizeComboBox(), settings,
				TrainerSettingsConstants.KEY_QUESTION_COMMENT_FONT_SIZE) {
			@Override
			public String getComponentValue() {
				return getFontSizeComboBoxValue(getComponent());
			}

			@Override
			protected void displayConfigValue() {
				setFontSizeComboBoxValue(getComponent(), getConfigValue());
			}
		};


		AbstractConfigComponent<JComboBox<String>> answersBox = new AbstractConfigComponent<JComboBox<String>>(
				buildFontSizeComboBox(), settings,
				TrainerSettingsConstants.KEY_ANSWERS_FONT_SIZE) {
			@Override
			public String getComponentValue() {
				return getFontSizeComboBoxValue(getComponent());
			}

			@Override
			protected void displayConfigValue() {
				setFontSizeComboBoxValue(getComponent(), getConfigValue());
			}
		};
		
		AbstractConfigComponent<JComboBox<String>> evaluationBox = new AbstractConfigComponent<JComboBox<String>>(
				buildFontSizeComboBox(), settings,
				TrainerSettingsConstants.KEY_EVALUATION_FONT_SIZE) {
			@Override
			public String getComponentValue() {
				return getFontSizeComboBoxValue(getComponent());
			}

			@Override
			protected void displayConfigValue() {
				setFontSizeComboBoxValue(getComponent(), getConfigValue());
			}
		};
		
//		final AbstractConfigComponent<JSpinner> maxCardBoxLevelBox = buildCardBoxSpinner(TrainerSettingsConstants.KEY_MAX_CARD_BOX_LEVEL);
//
		final AbstractConfigComponent<JSpinner> defaultCardBoxLevelBox = buildCardBoxSpinner(TrainerSettingsConstants.KEY_DEFAULT_CARD_BOX_LEVEL);

//		ChangeListener cardBoxSpinnerChangeListener = new ChangeListener() {
//			public void stateChanged(ChangeEvent e) {
//				updateSpinnerBounds(maxCardBoxLevelBox, defaultCardBoxLevelBox);
//			}
//		};

		/* initialises the spinner bounds */
//		updateSpinnerBounds(maxCardBoxLevelBox, defaultCardBoxLevelBox);
		int max = 4;
    setCardBoxSpinnerMax(defaultCardBoxLevelBox, max);

//		maxCardBoxLevelBox.getComponent().addChangeListener(
//				cardBoxSpinnerChangeListener);
//		defaultCardBoxLevelBox.getComponent().addChangeListener(
//				cardBoxSpinnerChangeListener);

		AbstractConfigComponent<JCheckBox> unansweredCardBox = new AbstractConfigComponent<JCheckBox>(
				new JCheckBox(), settings,
				TrainerSettingsConstants.KEY_USE_UNANSWERED_FOR_CARD_BOX) {
			@Override
			public String getComponentValue() {
				return getCheckBoxValue(getComponent());
			}

			@Override
			protected void displayConfigValue() {
				setCheckBoxValue(getComponent(), super.config
						.getBool(super.key));
			}
		};
		
		AbstractConfigComponent<JCheckBox> showCardBoxWarningsBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
		TrainerSettingsConstants.KEY_CARD_BOX_ACTIVE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
		
		AbstractConfigComponent<JCheckBox> categorySelectionMcExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_CATEGORY_SELECTION_MC_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> categorySelectionTextExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_CATEGORY_SELECTION_TEXT_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> examSelectionExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_EXAM_SELECTION_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> icuSelectionExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_ICU_SELECTION_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> icuExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_ICU_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> notesExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_NOTES_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> examModeExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_EXAM_MODE_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> icuFilterExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_ICU_FILTER_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> unansweredExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_UNANSWERED_FILTER_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> cardBoxFilterExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_CARD_BOX_FILTER_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
    
    AbstractConfigComponent<JCheckBox> textSearchExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_TEXT_SEARCH_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> historyExplanationBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_HISTORY_EXPLANATION_MESSAGE) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };
        
    AbstractConfigComponent<JCheckBox> tutorialOnStartupBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings,
        TrainerSettingsConstants.KEY_SHOW_TUTORIAL_ON_STARTUP) {
      @Override
      public String getComponentValue() {
        return getCheckBoxValue(getComponent());
      }

      @Override
      protected void displayConfigValue() {
        setCheckBoxValue(getComponent(), super.config
            .getBool(super.key));
      }
    };

    this.configCompHandler.addConfigComponent(questionCommentBox);
		this.configCompHandler.addConfigComponent(answersBox);
		this.configCompHandler.addConfigComponent(evaluationBox);
//		this.configCompHandler.addConfigComponent(maxCardBoxLevelBox);
		this.configCompHandler.addConfigComponent(defaultCardBoxLevelBox);
		this.configCompHandler.addConfigComponent(unansweredCardBox);
		this.configCompHandler.addConfigComponent(showCardBoxWarningsBox);
		this.configCompHandler.addConfigComponent(categorySelectionMcExplanationBox);
		this.configCompHandler.addConfigComponent(categorySelectionTextExplanationBox);
		this.configCompHandler.addConfigComponent(examSelectionExplanationBox);
		this.configCompHandler.addConfigComponent(icuSelectionExplanationBox);
		this.configCompHandler.addConfigComponent(icuExplanationBox);
		this.configCompHandler.addConfigComponent(notesExplanationBox);
		this.configCompHandler.addConfigComponent(examModeExplanationBox);
		this.configCompHandler.addConfigComponent(icuFilterExplanationBox);
		this.configCompHandler.addConfigComponent(unansweredExplanationBox);
		this.configCompHandler.addConfigComponent(cardBoxFilterExplanationBox);
		this.configCompHandler.addConfigComponent(textSearchExplanationBox);
		this.configCompHandler.addConfigComponent(historyExplanationBox);
		this.configCompHandler.addConfigComponent(tutorialOnStartupBox);

		FormLayout layout = new FormLayout(
				"right:max(180dlu;pref), 3dlu, right:pref,",// column
				""); // add rows dynamically
		DefaultFormBuilder builder = new DefaultFormBuilder(layout);
		builder.setDefaultDialogBorder();

		builder.appendSeparator(TrainerTextResources
				.getString("SETTINGS_DIALOG_FONT_SIZES"));

		builder.append(TrainerTextResources
				.getString("SETTINGS_DIALOG_QUESTION_COMMENT_FONT_SIZE"),
				questionCommentBox.getComponent());
		builder.nextLine();

		builder.append(TrainerTextResources
				.getString("SETTINGS_DIALOG_ANSWERS_FONT_SIZE"), answersBox
				.getComponent());
		builder.nextLine();
		
		builder.append(TrainerTextResources
				.getString("SETTINGS_DIALOG_EVALUATION_FONT_SIZE"),
				evaluationBox.getComponent());
		builder.nextLine();

		builder.appendSeparator(TrainerTextResources
				.getString("SETTINGS_DIALOG_CARD_BOX"));

//		builder.append(TrainerTextResources
//				.getString("SETTINGS_DIALOG_CARD_BOX_MAX_LEVEL"),
//				maxCardBoxLevelBox.getComponent());
//		builder.nextLine();

		builder.append(TrainerTextResources
				.getString("SETTINGS_DIALOG_CARD_BOX_DEFAULT_LEVEL"),
				defaultCardBoxLevelBox.getComponent());
		builder.nextLine();
		
		builder.append(TrainerTextResources
				.getString("SETTINGS_DIALOG_CARD_BOX_USE_UNANSWERED"),
				unansweredCardBox.getComponent());
		builder.nextLine();
		
    builder.append(TrainerTextResources
        .getString("SETTINGS_DIALOG_CARD_BOX_ACTIVE"),
        showCardBoxWarningsBox.getComponent());
    builder.nextLine();
    
    builder.appendSeparator(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_EXPLANATIONS"));
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_CATEGORY_SELECTION_MC_EXPLANATION"),
        categorySelectionMcExplanationBox.getComponent());
    builder.nextLine();

    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_CATEGORY_SELECTION_TEXT_EXPLANATION"),
        categorySelectionTextExplanationBox.getComponent());
    builder.nextLine();

    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_EXAM_SELECTION_EXPLANATION"),
        examSelectionExplanationBox.getComponent());
    builder.nextLine();

    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_ICU_SELECTION_EXPLANATION"),
        icuSelectionExplanationBox.getComponent());
    builder.nextLine();

    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_ICU_EXPLANATION"),
        icuExplanationBox.getComponent());
    builder.nextLine();
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_NOTES_EXPLANATION"),
        notesExplanationBox.getComponent());
    builder.nextLine();
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_EXAM_MODE_EXPLANATION"),
        examModeExplanationBox.getComponent());
    builder.nextLine();
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_ICU_FILTER_EXPLANATION"),
        icuFilterExplanationBox.getComponent());
    builder.nextLine();
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_UNANSWERED_EXPLANATION"),
        unansweredExplanationBox.getComponent());
    builder.nextLine();
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_CARD_BOX_FILTER_EXPLANATION"),
        cardBoxFilterExplanationBox.getComponent());
    builder.nextLine();
    
    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_TEXT_SEARCH_EXPLANATION"),
        textSearchExplanationBox.getComponent());
    builder.nextLine();

    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_SHOW_HISTORY_EXPLANATION"),
        historyExplanationBox.getComponent());
    builder.nextLine();

    builder.append(TrainerTextResources.getString("SETTINGS_DIALOG_TUTORIAL_ON_STARTUP"),
        tutorialOnStartupBox.getComponent());
    builder.nextLine();

		return builder.getPanel();
	}

	private JComboBox<String> buildFontSizeComboBox() {
		String[] items = {
		    TrainerTextResources.getString("SETTINGS_DIALOG_DEFAULT"),
		    "12", "13", "14", "16", "18", "20", "22", "24", "26", "28", "30", "32", "34", "36", "38", "40"
		};
		return new JComboBox<String>(items);
	}

	private String getFontSizeComboBoxValue(JComboBox<String> comboBox) {
		int index = comboBox.getSelectedIndex();

		if (index < 1) {
			return "-1";
		} else {
		  final String value = comboBox.getItemAt(index);
		  try {
		    Integer.parseInt(value);
		    return value;
		  } catch (NumberFormatException e) {
		    return "-1";
		  }
		}
	}

	private void setFontSizeComboBoxValue(JComboBox<String> comboBox, String configValue) {
	  int index = 0;
	  for (int i = 0; i < comboBox.getItemCount(); i++) {
	    if (configValue.equals(comboBox.getItemAt(i)) ) {
	      index = i;
	      break;
	    }
	  }
    comboBox.setSelectedIndex(index);
	}
	
	private AbstractConfigComponent<JSpinner> buildCardBoxSpinner(String key) {
		return new AbstractConfigComponent<JSpinner>(new JSpinner(
				new SpinnerNumberModel(this.config.getInt("maxCardBoxLevel") + 1,
						1, this.config.getInt("maxCardBoxLevel") + 1, 1)),
				settings, key) {
			@Override
			public String getComponentValue() {
				return (((Integer) getComponent().getValue()) - 1) + "";
			}

			@Override
			protected void displayConfigValue() {
				try {
					getComponent().setValue(Integer.parseInt(getConfigValue()) + 1);
				} catch (NumberFormatException e) {
          logger.error("Invalid configuration of setting parameter " + this.key + ": " + getConfigValue());
				} catch (IllegalArgumentException e) {
          logger.error("Invalid value of setting parameter " + this.key + ": " + getConfigValue());
				}
			}
		};
	}
	
	private void setCardBoxSpinnerMax(
			AbstractConfigComponent<JSpinner> spinner, int max) {
		if (spinner.getComponent().getModel() instanceof SpinnerNumberModel) {
			SpinnerNumberModel model = (SpinnerNumberModel) spinner
					.getComponent().getModel();
			if (model.getNumber().intValue() > max) {
				model.setValue(max);
			}
			model.setMaximum(max);
		}
	}
	
	private void setCardBoxSpinnerMin(
			AbstractConfigComponent<JSpinner> spinner, int min) {
		if (spinner.getComponent().getModel() instanceof SpinnerNumberModel) {
			SpinnerNumberModel model = (SpinnerNumberModel) spinner
					.getComponent().getModel();
			if (model.getNumber().intValue() < min) {
				model.setValue(min);
			}
			model.setMinimum(min);
		}
	}
	
	private void updateSpinnerBounds(
			AbstractConfigComponent<JSpinner> maxCardBoxLevelBox,
			AbstractConfigComponent<JSpinner> defaultCardBoxLevelBox) {
		int max = Integer.parseInt(maxCardBoxLevelBox.getComponentValue());
//		int def = Integer.parseInt(defaultCardBoxLevelBox.getComponentValue());

		setCardBoxSpinnerMax(defaultCardBoxLevelBox, max + 1);
		setCardBoxSpinnerMin(maxCardBoxLevelBox, 2/*Math.max(2, def + 1)*/);
		setCardBoxSpinnerMax(maxCardBoxLevelBox, 6);

	}
	
	private String getCheckBoxValue(JCheckBox checkBox) {
		return checkBox.isSelected() ? "true" : "false";
	}

	private void setCheckBoxValue(JCheckBox checkBox,
			boolean value) {
		checkBox.setSelected(value);
	}

}
