package de.kreawi.trainer.dialogs.onlineGame;

import de.kreawi.trainer.dialogs.onlineGame.custom_nodes.OGButton;
import de.kreawi.trainer.onlinegame.OnlineGameMain;
import de.kreawi.trainer.onlinegame.entities.OnlineGameUser;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.control.TextField;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;

public abstract class ProfileChange extends BorderPane {
  
  
  private final OnlineGameMain onlineGame;
  private final OnlineGameContainer onlineGameContainer;
  private final boolean edit;

  public ProfileChange(final OnlineGameMain onlineGame, final OnlineGameContainer onlineGameContainer,
      final boolean edit) {
    this.onlineGame = onlineGame;
    this.onlineGameContainer = onlineGameContainer;
    this.edit = edit;
    
    setCenter(getLogo());
    final Node signUpForm = getSignUpForm();
    BorderPane.setMargin(signUpForm, new Insets(0, 30, 50, 30));
    setBottom(signUpForm);
  }

  private Node getSignUpForm() {
    
    final TextField emailField = new TextField();
    emailField.setPromptText("Email");
    
    final TextField usernameField = new TextField();
    usernameField.setPromptText("Benutzername");
    
    final VBox vbox = new VBox(10, emailField, usernameField);
    vbox.setFillWidth(true);
    vbox.setAlignment(Pos.CENTER);
    
    if (edit) {
      
      final OnlineGameUser profile = this.onlineGame.getModel().getSettings().getProfileUser();
      emailField.setText(profile.getEmail());
      usernameField.setText(profile.getUserName());
      
      final Runnable updateAction = () -> {
        this.onlineGameContainer.editProfile(emailField.getText(), usernameField.getText());
      };
      final OGButton updateButton = new OGButton("Aktualisieren", "Profil aktualisieren", updateAction);
      
      final OGButton cancelButton = new OGButton("Abbrechen", "", () -> {
        this.onlineGameContainer.showDashboard();
      });
      
      final BorderPane buttonPane = new BorderPane();
      buttonPane.setLeft(cancelButton);
      buttonPane.setRight(updateButton);
      
      vbox.getChildren().add(buttonPane);
      
    } else {
      
      final Runnable signUpAction = () -> {
        this.onlineGameContainer.signup(emailField.getText(), usernameField.getText());
      };
      final OGButton signUpButton = new OGButton("Registrieren", "Diesen Benutzernamen registrieren", signUpAction);
      
      final OGButton helpButton = new OGButton("?", "Hilfe anzeigen", this.onlineGameContainer.getSignupHelpAction());
      
      vbox.getChildren().addAll(signUpButton, helpButton);
      
    }
    
    return vbox;
  }

  private Node getLogo() {
    Image image = new Image(getClass().getResourceAsStream("/de/kreawi/trainer/resources/images/logo_big.png"));
    final ImageView imageView = new ImageView(image);
    imageView.setFitWidth(150d);
    // TODO: binding crashes layout
//    imageView.fitWidthProperty().bind(widthProperty().multiply(.5d));
    imageView.setPreserveRatio(true);
    return imageView;
  }
  
}
