package de.kreawi.trainer.dialogs.onlineGame.custom_nodes;

import java.io.IOException;
import java.io.InputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.kreawi.trainer.onlinegame.entities.OnlineGameUser;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.geometry.Insets;
import javafx.geometry.Rectangle2D;
import javafx.scene.SnapshotParameters;
import javafx.scene.SnapshotResult;
import javafx.scene.effect.BlurType;
import javafx.scene.effect.DropShadow;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.util.Callback;

public class ProfileImage extends StackPane {
  
  private static final double IMAGE_SIZE = 50d;
  private static final double SHADOW_SIZE = 10d;
  
  private static final String NO_PROFILE_IMAGE_PATH = "/de/kreawi/trainer/resources/images/noProfileImage.png";
  private static Image CACHED_NO_PROFILE_IMAGE;

  private static final Logger logger = LoggerFactory.getLogger(ProfileImage.class); 

  public ProfileImage(final OnlineGameUser user) {
    this(user, false);
  }

  public ProfileImage(final OnlineGameUser user, final boolean withShadow) {
    init(getImageOrDummy(user), withShadow);
  }
  
  private void init(final Image image, final boolean withShadow) {
    final ImageView imageView = new ImageView(image);
    final double circleRadius = IMAGE_SIZE / 2;
    final Circle clip = new Circle(circleRadius, circleRadius, circleRadius);
    imageView.setClip(clip);
    
    final Runnable setViewport = () -> {
      final double imageWidth = image.getWidth();
      final double imageHeight = image.getHeight();
      final double x;
      final double y;
      final double width;
      final double height;
      // make square format
      if (imageWidth > imageHeight) {
        width = imageHeight;
        x = (imageWidth - width) / 2d;
        height = imageHeight;
        y = 0;
      } else {
        height = imageWidth;
        y = (imageHeight - height) / 2d;
        x = 0;
        width = imageWidth;
      }
      Rectangle2D croppedPortion = new Rectangle2D(x, y, width, height);
      imageView.setViewport(croppedPortion);
      imageView.snapshot(new Callback<SnapshotResult, Void>() {
        @Override
        public Void call(SnapshotResult param) {
          Platform.runLater(() -> {
            imageView.setImage(param.getImage());
            imageView.setViewport(null);
          });
          return null;
        }
        
      }, new SnapshotParameters(), null);
    };
    if (image.isBackgroundLoading()) {
      image.progressProperty().addListener(new ChangeListener<Number>() {
        @Override
        public void changed(ObservableValue<? extends Number> observable, Number oldValue, Number progress) {
          if ((Double) progress == 1.0) {
            Platform.runLater(() -> {
              setViewport.run();
            });
          }
        }
      });
    } else {
      setViewport.run();
    }
    imageView.setFitWidth(IMAGE_SIZE);
    imageView.setFitHeight(IMAGE_SIZE);
    imageView.setPreserveRatio(true);
    imageView.setStyle("-fx-background-color: gray");
    getChildren().add(imageView);
    
    if (withShadow) {
      final Circle circle = new Circle(IMAGE_SIZE / 2d);  // Color.web("black", .5)
      circle.setStyle("-fx-background-color: gray");
      circle.setEffect(new DropShadow(BlurType.THREE_PASS_BOX, Color.GRAY, SHADOW_SIZE, 0d, 0d, 0d));
      
      StackPane.setMargin(imageView, new Insets(SHADOW_SIZE));
      StackPane.setMargin(circle, new Insets(SHADOW_SIZE));
      
      getChildren().add(0, circle);
    }

  }

  private Image getImageOrDummy(final OnlineGameUser user) {
    final Image image;
    if (user.getProfileImage() == null || "".equals(user.getProfileImage())) {
      if (CACHED_NO_PROFILE_IMAGE == null) {
        try (final InputStream is = this.getClass().getResourceAsStream(NO_PROFILE_IMAGE_PATH)) {
          CACHED_NO_PROFILE_IMAGE = new Image(is);
        } catch (IOException e) {
          logger.error("Error on reading Placeholder image! (Path: " + NO_PROFILE_IMAGE_PATH + ")", e);
          throw new RuntimeException(e);
        }
      }
      image = CACHED_NO_PROFILE_IMAGE;
    } else {
      image = new Image(user.getProfileImage(), true);
    }
    return image;
  }
}
