package de.kreawi.trainer.dialogs.selection;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jgoodies.uif_lite.component.Factory;
import com.jquizengine.questions.Category;
import com.jquizengine.questions.Exam;
import com.jquizengine.questions.TitledQuestionGroup;
import com.jquizengine.questions.collection.QuestionPool;

import de.kreawi.trainer.TrainerFrame;
import de.kreawi.trainer.TrainerFrame.Actions;
import de.kreawi.trainer.TrainerModel;
import de.kreawi.trainer.TrainerSettingsConstants;
import de.kreawi.trainer.dialogs.KreawiInternalFrame;
import de.kreawi.trainer.util.TrainerTextResources;

@SuppressWarnings("serial")
public class ExamSelection extends AbstractSelectionDialog {

	private JTree tree;
	private Map<Exam, Map<Category, Integer>> examCategoryCountCache;
	
	private static final Logger logger = LoggerFactory.getLogger(ExamSelection.class); 

  public ExamSelection(TrainerFrame mainframe, TrainerModel model, Map<Actions, Action> actionMap,
      QuestionPool questionPool) {
		super(mainframe, model, actionMap, questionPool);
		
		updateExamCategoryCountCache();
		
	}

  @Override
	protected boolean showCardBoxPanel() {
	  return false;
	}
  
  @Override
  protected boolean showCardBoxFilterOption() {
    return !isTextQuestionPool();
  }

  @Override
  protected boolean showExamMode() {
    return !isTextQuestionPool();
  }

  @Override
  protected boolean showICU() {
    return true;
  }

  @Override
  protected boolean isTotalDependsOnFilter() {
    return false;
  }

  @Override
  protected boolean showCategoryColors() {
    // TODO Auto-generated method stub
    return false;
  }
	
  @Override
  protected String getExplanationMessageKey() {
    return TrainerSettingsConstants.KEY_SHOW_EXAM_SELECTION_EXPLANATION_MESSAGE;
  }

	@Override
	protected KreawiInternalFrame buildSelectionPanel() {
		KreawiInternalFrame kif = new KreawiInternalFrame(TrainerTextResources
				.getString("EXAM_SELECTION_TITLE_EXAMS"), new BorderLayout());

		JPanel gradientPanel = kif.getContent();
		gradientPanel.setBorder(BorderFactory.createEmptyBorder());

		this.tree = getExamsTree();
		this.tree.setBorder(BorderFactory.createEmptyBorder(2, 4, 0, 2));
		this.tree.setRootVisible(true);

		for (int i = 0; i < this.tree.getRowCount(); i++) {

			Object obj = this.tree.getPathForRow(i).getLastPathComponent();

			if (obj instanceof DefaultMutableTreeNode) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) obj;
				if (node.getUserObject() instanceof String) {
					this.tree.expandRow(i);
				}
			}

		}

		this.tree.setSelectionModel(new LeafSelectionModel());
		this.tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		this.tree.addTreeSelectionListener(new TreeSelectionListener() {

			public void valueChanged(TreeSelectionEvent e) {
			   TreePath newPath = e.getNewLeadSelectionPath();

		    tree.scrollPathToVisible(newPath);
				selectionChanged();
			}

		});
		
		this.tree.addTreeWillExpandListener(new TreeWillExpandListener() {
      public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
      }
      
      public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
        if (event.getPath().getParentPath() == null) {
          // don't collapse root node
          throw new ExpandVetoException(event);
        }
      }
    });

		this.tree.setCellRenderer(new DefaultTreeCellRenderer() {
			public Component getTreeCellRendererComponent(final JTree tree,
					Object value, final boolean selected, final boolean expanded,
					boolean leaf, final int row, final boolean hasFocus) {
				
        boolean isEnabled = true;

				if (value instanceof DefaultMutableTreeNode) {
          DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
				  
          if (node.getUserObject() instanceof TitledQuestionGroup) {
            int questionCount = 0;
            String valueString = "";
            if (node.getUserObject() instanceof Exam) {
              Exam exam = (Exam)node.getUserObject();
              questionCount = getChoosableQuestions().getQuestionCountForGroup(exam);
              
              valueString  = exam.getTitle();
              
              leaf = true;
            } else if (node.getUserObject() instanceof Category) {
              Category category = (Category) node.getUserObject();
              if (node.getParent() instanceof DefaultMutableTreeNode) {
                DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node
                .getParent();
                if (parent.getUserObject() instanceof Exam) {
                  Exam exam = (Exam) parent.getUserObject();
                  questionCount = examCategoryCountCache.get(exam).get(category);

                  valueString = category.getTitle();
                }
              }
            }
            valueString += " (" + questionCount + ")";
            value = valueString;
            isEnabled = !(questionCount < 1);
          }
				}
				
        DefaultTreeCellRenderer comp = (DefaultTreeCellRenderer) super.getTreeCellRendererComponent(tree, value,
            selected, expanded, leaf, row, hasFocus);

				comp.setEnabled(isEnabled);

				comp.setBackgroundNonSelectionColor(new Color(0, 0, 0, 0));
				
				return comp;
			}

		});

		if (Boolean.TRUE.equals(UIManager.getDefaults().get(
				KreawiInternalFrame.USE_GRADIENT_BACKGROUND_KEY))) {
			this.tree.setOpaque(false);
		}

		ToolTipManager.sharedInstance().registerComponent(this.tree);
		this.tree.setToolTipText(TrainerTextResources
				.getString("SELECTION_LABEL_EXAM_SELECTION_EXPLANATION"));

		JScrollPane scrollPane = Factory.createStrippedScrollPane((this.tree));
		scrollPane.setOpaque(false);
		scrollPane.getViewport().setOpaque(false);

		gradientPanel.add(scrollPane);

		return kif;
	}
	
	private JTree getExamsTree() {

    DefaultMutableTreeNode root = new DefaultMutableTreeNode();

    final ArrayList<Exam> sortedExams = new ArrayList<Exam>(getAllExams());
    if (isTextQuestionPool()) {
      Collections.sort(sortedExams, new Comparator<Exam>() {
        public int compare(Exam e1, Exam e2) {
          return e1.getTitle().compareToIgnoreCase(e2.getTitle());
        }
      });
    } else {
      Collections.sort(sortedExams, new Comparator<Exam>() {
        public int compare(Exam e1, Exam e2) {
          int result = getYear(e1).compareTo(getYear(e2)) * -1;
          if (result == 0) {
            if ("Oktober".equals(getMonth(e1))) {
              result = -1;
            }
          }
          return result;
        }
        private String getYear(final Exam exam) {
          final String[] splittedTitle = exam.getTitle().split(" ");
          return splittedTitle[splittedTitle.length - 1];
        }
        private String getMonth(final Exam exam) {
          final String[] splittedTitle = exam.getTitle().split(" ");
          return splittedTitle[splittedTitle.length - 2];
        }
      });
    }
    
    for (Exam exam : sortedExams) {
      DefaultMutableTreeNode examNode = new DefaultMutableTreeNode(exam);
      root.add(examNode);
      
      ArrayList<Category> categories = new ArrayList<Category>(getCategoriesForExam(exam));
      Collections.sort(categories);
      
      /*
       * add categories as child nodes, so users can check which
       * categories where used for an exam
       */
      for (Category cat : categories) {
        DefaultMutableTreeNode catNode = new DefaultMutableTreeNode(cat);
        examNode.add(catNode);
      }

    }

    return new JTree(root) {

      @Override
      public String getToolTipText(MouseEvent event) {
        return (String)super.getClientProperty(TOOL_TIP_TEXT_KEY);
      }
      
    };

  }
	
	@Override
	final protected void choosableQuestionsChanged() {
	  updateExamCategoryCountCache();
	  this.tree.revalidate();
	  this.tree.repaint();
	}
	
	@Override
  final protected QuestionPool getSelectedQuestions() {
    
    if (this.tree.getSelectionPath() != null) {
      
      Object obj = this.tree.getSelectionPath().getLastPathComponent();
      
      if (obj instanceof DefaultMutableTreeNode) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode) obj;
        Object userObject = node.getUserObject();
        if (userObject instanceof Exam) {
          Exam exam = (Exam) userObject;
          return getChoosableQuestions().getQuestionPoolForExam(exam);
        } else if (userObject instanceof Category) {
          Category category = (Category) userObject;
          Exam exam = (Exam) ((DefaultMutableTreeNode) node.getParent()).getUserObject();
          return getChoosableQuestions().getQuestionPoolForExam(exam).getQuestionPoolForCategory(category);
        }
      }
      
    }
    
    return null;
    
  }
	
	protected int getSelectedQuestionsCount() {

	  QuestionPool selectedQuestions = getSelectedQuestions();

    if (selectedQuestions == null) {
      return 0;
    } else {
      return selectedQuestions.getQuestionCount();
    }
    
	}
	
	private void updateExamCategoryCountCache() {
		
		this.examCategoryCountCache = new HashMap<Exam, Map<Category, Integer>>();

		for (Exam exam : getAllExams()) {
			
			Map<Category, Integer> categoryMap = new HashMap<Category, Integer>();
			
			final Set<Category> categories = getCategoriesForExam(exam);
			
			final QuestionPool choosableQuestionsForExam = getChoosableQuestions().getQuestionPoolForExam(exam);

			for (Category category : categories) {

				int count = choosableQuestionsForExam.getQuestionCountForCategory(category);

				categoryMap.put(category, count);
				
			}
			
			this.examCategoryCountCache.put(exam, categoryMap);
		}
		
	}

	/*
	 * Only leaves should be selectable in the tree, so we have to filter out
	 * TreePaths which don't lead to a leaf.
	 */
	private static class LeafSelectionModel extends DefaultTreeSelectionModel {

		@Override
		public void setSelectionPaths(TreePath[] pPaths) {
			super.setSelectionPaths(getLeafTreePaths(pPaths));
		}

		@Override
		public void addSelectionPaths(TreePath[] paths) {
			super.addSelectionPaths(getLeafTreePaths(paths));
		}

		/*
		 * TreePath objects, which don't lead to a leaf are filtered out.
		 */
		private TreePath[] getLeafTreePaths(TreePath[] pPaths) {
			ArrayList<TreePath> retPaths = new ArrayList<TreePath>();

			for (TreePath tp : pPaths) {
				Object obj = tp.getLastPathComponent();
				if (obj instanceof DefaultMutableTreeNode) {
					DefaultMutableTreeNode node = (DefaultMutableTreeNode) obj;
					if (node.getUserObject() instanceof Exam) {
						retPaths.add(tp);
					} else if (node.getParent() instanceof DefaultMutableTreeNode) {
						retPaths.add(tp);
//						DefaultMutableTreeNode parent = (DefaultMutableTreeNode) node
//								.getParent();
//						if (parent.getUserObject() instanceof Exam) {
//							retPaths.add(tp.getParentPath());
//						}
					}
				}
			}

			return retPaths.toArray(new TreePath[retPaths.size()]);

		}

	}

}
