package de.kreawi.trainer.quiz;

import java.util.HashSet;
import java.util.Set;

import com.jquizengine.questions.Category;
import com.jquizengine.questions.Exam;
import com.jquizengine.questions.Question;
import com.jquizengine.questions.collection.ArrayListQuestionPool;

import de.kreawi.trainer.util.TrainerTextResources;

public class KreawiQuizHelper {
  
  public static String getTitle(final KreawiQuiz quiz) {
    if (quiz.isCardBoxSelection()) {
      return TrainerTextResources.getString("QUIZ_TITLE_CARD_BOX") + " " + (quiz.cardBoxLevel() + 1);
    } else {
      return getCategoryExam(quiz);
    }
  }

  private static String getCategoryExam(final KreawiQuiz quiz) {

    ArrayListQuestionPool questionPool = new ArrayListQuestionPool(quiz.getQuestions());

    Set<Category> categories = questionPool.getCategories();
    Set<Exam> exams = questionPool.getExams();

    if (exams.size() == 1) {
      Exam exam = exams.iterator().next();

      if (questionPool.getQuestionCount() == questionPool
          .getQuestionPoolForExam(exam).getQuestionCount()) {
        return exam.getTitle();
      }

    }

    if (exams.size() > 1) {

      Set<Question> questions = new HashSet<Question>();

      for (Exam exam : exams) {
        questions.addAll(questionPool.getQuestionPoolForExam(exam)
            .getQuestions());
      }

      if (questionPool.getQuestionCount() == questions.size()) {
        return TrainerTextResources.getStringWithSubstitution(
            "QUIZ_TITLE_SEVERAL_EXAMS", exams.size() + "");
      }
    }

    if (exams.size() > 0) {
      String[] sizes = { categories.size() + "", exams.size() + "" };
      return TrainerTextResources.getStringWithSubstitution(
          "QUIZ_TITLE_SEVERAL_CATEGORIES_EXAMS", sizes);
    }

    if (categories.size() > 1) {
      return TrainerTextResources.getStringWithSubstitution(
          "QUIZ_TITLE_SEVERAL_CATEGORIES", categories.size() + "");
    }

    if (categories.size() == 1) {
      return categories.iterator().next().getTitle();
    }
    
    if (questionPool.getQuestionCount() == 0) {
      return TrainerTextResources.getString("QUIZ_TITLE_NO_QUESTIONS_ERROR");
    }

    return TrainerTextResources.getString("ERROR");
  }

  public static Double getProgress(final KreawiQuiz quiz){
    int all = quiz.getQuestionCount();
    int answered = quiz.getAnsweredQuestionCount();

    double percentAnswered = all > 0 ? (double) answered / (double) all : 0;
    return percentAnswered;
  }
  
  public static Double getCorrect(final KreawiQuiz quiz) {
    int all = quiz.getQuestionCount();
    int correct = quiz.getCorrectAnswersCount();

    double percentCorrectAnswered = all > 0 ? (double) correct / (double) all : 0d;

    return quiz.isExamMode() && quiz.hasUnanswered(false) ? null : percentCorrectAnswered;
  }

}
