package de.kreawi.trainer.util;

import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.JPanel;
import javax.swing.Timer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class FadeInPanel extends JPanel implements ComponentListener {
  
  private BufferedImage image;
  
  private int scaledImageWidth;
  private int scaledImageHeight;
  private int offsetX = 0;
  private int offsetY = 0;

  private float opacity = 0.0f;

  private final Timer t;

  private final static int TIMER_DELAY = 50;
  
  private long expectedDuration;

  private static final Logger logger = LoggerFactory.getLogger(FadeInPanel.class); 

  public FadeInPanel(final URL imageUrl, final long initialExpectedDuration) {
    
    this.expectedDuration = initialExpectedDuration;
    
    setOpaque(false);
    try {
        image = ImageIO.read(imageUrl);
    } catch (IOException e) {
        e.printStackTrace();
    }
    
    t = new Timer(TIMER_DELAY, new ActionListener() {
      private long startTime = 0l;

      public void actionPerformed(ActionEvent e) {
        if (startTime == 0l) {
          startTime = System.currentTimeMillis();
        }
        final long timePassed = System.currentTimeMillis() - startTime;
        if (timePassed > expectedDuration) {
          opacity = 1f;
          t.stop();
        } else {
          opacity = ((float) timePassed / (float) expectedDuration);
        }
        repaint();
      }
    });
    t.setInitialDelay(0);
    
    this.addComponentListener(this);
  }
  
  public void setVisible(final boolean visible) {
    super.setVisible(visible);
    if (visible) {
      t.start();
    } else {
      t.stop();
    }
  }
  
  public void paintComponent(Graphics g) {
    super.paintComponent(g);
    Graphics2D g2d = (Graphics2D) g;
    RenderingHints rh = new RenderingHints(
        RenderingHints.KEY_ANTIALIASING, 
        RenderingHints.VALUE_ANTIALIAS_ON
        );
    g2d.setRenderingHints(rh);
    g2d.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, opacity));
    g2d.drawImage(image, offsetX, offsetY, scaledImageWidth, scaledImageHeight, null);
  }

  public void componentResized(ComponentEvent e) {
    final double preferredScaleFactorX = (double) getWidth() / (double) image.getWidth();
    final double preferredScaleFactorY = (double) getHeight() / (double) image.getHeight();
    final double scaleFactor = Math.min(preferredScaleFactorX, preferredScaleFactorY);
    scaledImageWidth = (int) (scaleFactor * (double) image.getWidth());
    scaledImageHeight = (int) (scaleFactor * (double) image.getHeight());
    offsetX = (int) ((double) (getWidth() - scaledImageWidth) / 2.0);
    offsetY = (int) ((double) (getHeight() - scaledImageHeight) / 2.0);
  }

  public void componentMoved(ComponentEvent e) {
  }

  public void componentShown(ComponentEvent e) {
  }

  public void componentHidden(ComponentEvent e) {
  }
  
}
