package de.kreawi.trainer.util;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.martinknauer.util.config.AbstractConfigComponent;
import de.martinknauer.util.config.Config;
import de.martinknauer.util.config.ConfigComponentHandler;

public class TrainerUtil {
	
  private static final Logger logger = LoggerFactory.getLogger(TrainerUtil.class); 

  public static void showDeactivateableDialog(final String title, final String message,
      final String showAgainSettingsKey, final Config settings, final Component parent) {
    final AbstractConfigComponent<JCheckBox> showCardBoxWarningsBox = new AbstractConfigComponent<JCheckBox>(
        new JCheckBox(), settings, showAgainSettingsKey) {
      @Override
      public String getComponentValue() {
        return (getComponent().isSelected() ? "true" : "false");
      }
      @Override
      protected void displayConfigValue() {
        boolean oldValue = super.config.getBool(super.key);
        getComponent().setSelected(oldValue);
      }
    };
    final ConfigComponentHandler configCompHandler = new ConfigComponentHandler();
    configCompHandler.addConfigComponent(showCardBoxWarningsBox);
    final Box configPanel = Box.createHorizontalBox();
    configPanel.add(new JLabel(TrainerTextResources.getString("SELECTION_LABEL_CARD_BOX_WARNINGS_SHOW_MESSAGE")));
    configPanel.add(Box.createRigidArea(new Dimension(5,0)));
    final JCheckBox checkbox = showCardBoxWarningsBox.getComponent();
    configPanel.add(checkbox);
    configPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
    configPanel.setMaximumSize(configPanel.getPreferredSize());
    configPanel.setMinimumSize(configPanel.getPreferredSize());

    final JTextPane textPane = TrainerHtmlUtil.createHtmlPane(message);
    textPane.setAlignmentX(Component.LEFT_ALIGNMENT);
    textPane.setMaximumSize(textPane.getPreferredSize());
    textPane.setMinimumSize(textPane.getPreferredSize());

    final Box dialogPanel = Box.createVerticalBox();
    dialogPanel.add(textPane);
    dialogPanel.add(Box.createRigidArea(new Dimension(0, 8)));
    dialogPanel.add(configPanel);
    dialogPanel.add(Box.createRigidArea(new Dimension(0, 15)));
    
    JOptionPane.showMessageDialog(parent, dialogPanel, title, JOptionPane.INFORMATION_MESSAGE);
    configCompHandler.commit();
	}
  
  public static void writeObjectToFile(final Object o, final File file) throws FileNotFoundException, IOException {
    try (
        FileOutputStream fos = new FileOutputStream(file);
        ObjectOutputStream oos = new ObjectOutputStream(fos);
        ) {
      oos.writeObject(o);
    }
  }

  public static Object readObjectFromFile(final File file)
      throws FileNotFoundException, IOException, ClassNotFoundException {
    try (
        FileInputStream fis = new FileInputStream(file);
        ObjectInputStream ois = new ObjectInputStream(fis);
        ) {
      return ois.readObject();
    }
  }

  /**
   * Read defaults from internal file and then override with values from external file
   * 
   * @param file
   * @param internalFilePath
   * @return
   */
  public static Properties loadPropertiesFile(final File file, final String internalFilePath) {
    final Properties defaults = new Properties();
    if (internalFilePath != null) {
      try (final InputStream in = TrainerUtil.class.getClassLoader().getResourceAsStream(internalFilePath)) {
        defaults.loadFromXML(in);
      } catch (Exception e) {
        logger.info("Could not read defaults from internal file (" + internalFilePath + "). Using empty defaults.");
      }
    }
    return loadPropertiesFile(file, defaults);
  }
  
  /**
   * Use defaults and override with values form external file
   * 
   * @param file
   * @param defaults (may be null)
   * @return
   */
  public static Properties loadPropertiesFile(final File file, final Properties defaults) {
    final Properties properties;
    if (defaults == null) {
      properties = new Properties();
    } else {
      properties = new Properties(defaults);
    }
    try (FileInputStream fis = new FileInputStream(file)) {
      properties.loadFromXML(fis);
    } catch (Exception e) {
      logger.info("Could not read properties from file (" + file.getAbsolutePath() + "). Reading defaults...");
    }
    return properties;
  }

  public static void storePropertiesFile(final File file, Properties properties)
      throws FileNotFoundException, IOException {
    if (!file.exists()) {
      logger.info("File " + file.getName() + " not found. A new one will be created.");
    }
    try (final FileOutputStream fos = new FileOutputStream(file)) {
      properties.storeToXML(fos, null);
    }
  }
  
  private static final Thread DUMMY_HOOK = new Thread();

  public static boolean isShuttingDown() {
    try {
      Runtime.getRuntime().addShutdownHook(DUMMY_HOOK);
      Runtime.getRuntime().removeShutdownHook(DUMMY_HOOK);
    } catch (IllegalStateException e) {
      return true;
    }
    
    return false;
  }
  
}
